@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2011 by Barry A. Warsaw
+# Copyright (C) 2004-2012 by Barry A. Warsaw
 #
 # This file is part of flufl.enum
 #
@@ -16,24 +16,25 @@
 
 """Python enumerations."""
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import absolute_import, print_function, unicode_literals
 
 __metaclass__ = type
 __all__ = [
-    b'Enum',
-    b'make_enum',
+    'Enum',
+    'make_enum',
     ]
 
 
-COMMASPACE = ', '
-SPACE = ' '
-
 import re
 import warnings
 
 
+COMMASPACE = ', '
+SPACE = ' '
+IDENTIFIER_RE = r'[a-zA-Z_][a-zA-Z0-0_]*'
+
+
 
-# pylint: disable-msg=C0203
 class EnumMetaclass(type):
     """Meta class for Enums."""
 
@@ -53,7 +54,6 @@ class EnumMetaclass(type):
         # conversion from integer).
         for basecls in cls.__mro__:
             if hasattr(basecls, '_enums'):
-                # pylint: disable-msg=W0212
                 cls._enums.update(basecls._enums)
         # For each class attribute, create an EnumValue and store that back on
         # the class instead of the int.  Skip Python reserved names.  Also add
@@ -75,8 +75,9 @@ class EnumMetaclass(type):
         raise AttributeError(name)
 
     def __repr__(cls):
-        enums = ['%s: %d' % (cls._enums[k], k) for k in sorted(cls._enums)]
-        return '<%s {%s}>' % (cls.__name__, COMMASPACE.join(enums))
+        enums = ['{0}: {1:d}'.format(
+            cls._enums[k], k) for k in sorted(cls._enums)]
+        return '<{0} {{{1}}}>'.format(cls.__name__, COMMASPACE.join(enums))
 
     def __iter__(cls):
         for i in sorted(cls._enums):
@@ -110,11 +111,11 @@ class EnumValue:
         self._name = name
 
     def __repr__(self):
-        return '<EnumValue: %s.%s [int=%d]>' % (
+        return '<EnumValue: {0}.{1} [int={2:d}]>'.format(
             self._enum.__name__, self._name, self._value)
 
     def __str__(self):
-        return '%s.%s' % (self._enum.__name__, self._name)
+        return '{0}.{1}'.format(self._enum.__name__, self._name)
 
     def __int__(self):
         return self._value
@@ -164,17 +165,74 @@ class EnumValue:
 
 
 
-class Enum:
-    """The public API Enum class."""
+# Define the Enum class using metaclass syntax compatible with both Python 2
+# and Python 3.
+Enum = EnumMetaclass(str('Enum'), (), {
+    '__doc__': 'The public API Enum class.',
+    })
 
-    # pylint: disable-msg=W0232
-    __metaclass__ = EnumMetaclass
+
+
+def make(name, source):
+    """Return an Enum class from a name and source.
+
+    This is a convenience function for defining a new enumeration given an
+    existing sequence.  When an sequence is used, it is iterated over to get
+    the enumeration value items.  The sequence iteration can either return
+    strings or 2-tuples.  When strings are used, values are automatically
+    assigned starting from 1.  When 2-tuples are used, the first item of the
+    tuple is a string and the second item is the integer value.
+
+    `source` must be homogeneous.  You cannot mix string-only and 2-tuple
+    items in the sequence.
+
+    :param name: The resulting enum's class name.
+    :type name: byte string (or ASCII-only unicode string)
+    :param source: An object giving the enumeration value items.
+    :type source: A sequence of strings or 2-tuples.
+    :return: The new enumeration class.
+    :rtype: instance of `EnumMetaClass`
+    :raises ValueError: when a heterogeneous source is given, or when
+        non-identifiers are used as enumeration value names.
+    """
+    namespace = {}
+    illegals = []
+    have_strings = None
+    try:
+        # Python 2
+        string_type = basestring
+    except NameError:
+        # Python 3
+        string_type = str
+    for i, item in enumerate(source, start=1):
+        if isinstance(item, string_type):
+            if have_strings is None:
+                have_strings = True
+            elif not have_strings:
+                raise ValueError('heterogeneous source')
+            namespace[item] = i
+            if re.match(IDENTIFIER_RE, item) is None:
+                illegals.append(item)
+        else:
+            if have_strings is None:
+                have_strings = False
+            elif have_strings:
+                raise ValueError('heterogeneous souce')
+            item_name, item_value = item
+            namespace[item_name] = item_value
+            if re.match(IDENTIFIER_RE, item_name) is None:
+                illegals.append(item_name)
+    if len(illegals) > 0:
+        raise ValueError('non-identifiers: {0}'.format(SPACE.join(illegals)))
+    return EnumMetaclass(str(name), (Enum,), namespace)
 
 
 
 def make_enum(name, value_string, iterable=None):
     """Return an Enum class from a name and a value string.
 
+    *This function is deprecated; use `make()` instead.*
+
     This is a convenience function for defining a new enumeration when you
     don't care about the values of the items.  The values are automatically
     created by splitting the value string on spaces.
@@ -193,9 +251,11 @@ def make_enum(name, value_string, iterable=None):
     :return: The new enumeration class.
     :rtype: instance of `EnumMetaClass`
     """
+    warnings.warn('make_enum() is deprecated; use make() instead',
+                  DeprecationWarning)
     value_names = value_string.split()
     illegals = [value for value in value_names
-                if re.match(r'[a-zA-Z_][a-zA-Z0-9_]*', value) is None]
+                if re.match(IDENTIFIER_RE, value) is None]
     if len(illegals) > 0:
         raise ValueError('non-identifiers: {0}'.format(SPACE.join(illegals)))
     if iterable is None:
